#!BPY

"""
Name: 'Blender Python (.py)...'
Blender: 240
Group: 'Export'
Tooltip: 'Generates a python script recreating selected meshes'
"""

__author__ = " Jean-Baptiste PERIN (jb_perin(at)yahoo.fr)"
__version__ = "1.0"
__url__ = ["blender", "elysiun",
"PYTHON EXPORTER , http://www.zoo-logique.org/3D.Blender/index.php3?zoo=dld&rep=zip ",
"Communicate problems and errors, http://www.zoo-logique.org/3D.Blender/newsportal/thread.php?group=3D.Blender"]
__email__= ["M. PERIN Jean-Baptiste, jb_perin@yahoo.fr", "scripts"]
__bpydoc__ = """\
python_export.py 

Script generating a python script which build selected meshes,  
Usage:<br>
 - Select meshes you want to export <br>
 - Launch this script (Alt-P);<br>
A python script is generated that build these meshes.
Generated scripts are stored into <blender_exe_path>/.blender/scripts directory<br>
or in the user defined Pyhton directory if setted up <br>
(in that case .. created script won't appear in Add menu)<br>
Generated script can then be launched from menu Script->Add->your_mesh_name<br>
Python output takes into account: <br>
- vertices coordinates, uv coordinates <br>
- faces'modes, transp and mat (I was unable to deal with flags ..)<br>
- mesh material (not object material)
- object position (relatively to 3D cursor), size and rotation <br>
- object and mesh name <br>
- mesh's vertex group <br>
Todo:
- handle object materials
- handle target directory error
- handle texture
"""

# -------------------------------------------------------------------------- 
# python_export.py 
# -------------------------------------------------------------------------- 
# ***** BEGIN GPL LICENSE BLOCK ***** 
# 
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License 
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
# 
# ***** END GPL LICENCE BLOCK ***** 
# -------------------------------------------------------------------------- 


import Blender
from Blender import *

user_script_dir = Blender.Get('uscriptsdir')
#print user_script_dir
if user_script_dir != None:
  blender_script_dir = user_script_dir 
else: 
  blender_path = Blender.sys.progname
  blender_install_dir = sys.dirname(sys.splitext((blender_path))[0])
  blender_script_dir = blender_install_dir+sys.sep +'.blender'+sys.sep +'scripts'

#blender_script_dir = blender_install_dir+sys.sep +'.blender'+sys.sep +'scripts'



def exportMaterial (themat, lescript, meshradical, matnumber):
#####  shader specific

###applies to the "Minnaert" Diffuse Shader only)
##"lemat.setDiffuseDarkness(%f)\n"%lemat.getDiffuseDarkness()  
###applies to the "Toon" Diffuse Shader only
##"lemat.setDiffuseSmooth(%f)\n"%lemat.getDiffuseSmooth()  
###applies to the "Blinn" Specular Shader only
##"lemat.setRefracIndex(%f)\n"%lemat.getRefracIndex()  
###applies to the "Toon" Diffuse Shader only
##"lemat.setSpecSize(%f)\n"%lemat.getSpecSize()  
###applies to the "Toon" Specular Shader only
##"lemat.setSpecSmooth(%f)\n"%lemat.getSpecSmooth()  
	lemat = themat
	
	
	matvar = "lemat%d"%matnumber
	lescript.write("%s = GetOrCreateMaterial('%s')\n"%(matvar, lemat.getName())) ## computename ???
	lescript.write("%s.setAdd(%f)\n"%(matvar,lemat.getAdd()))
	lescript.write("%s.setAlpha(%f)\n"%(matvar,lemat.getAlpha()))
	lescript.write("%s.setAmb(%f)\n"%(matvar,lemat.getAmb())) 
	#from one of the shaders in Material.Shaders dict
	lescript.write("%s.setDiffuseShader(%d)\n"%(matvar,lemat.getDiffuseShader()))  
	lescript.write("%s.setEmit(%f)\n"%(matvar,lemat.getEmit()) ) 
	#when transparent raytrace is enabled 
	lescript.write("%s.setFilter(%f)\n"%(matvar,lemat.getFilter()))  
	lescript.write("%s.setFlareBoost(%f)\n"%(matvar,lemat.getFlareBoost()))  
	#offset in the seed table
	lescript.write("%s.setFlareSeed(%d)\n"%(matvar,lemat.getFlareSeed()))  
	lescript.write("%s.setFlareSize(%f)\n"%(matvar,lemat.getFlareSize()))  
	lescript.write("%s.setFresnelMirr(%f)\n"%(matvar,lemat.getFresnelMirr()))  
	lescript.write("%s.setFresnelMirrFac(%f)\n"%(matvar,lemat.getFresnelMirrFac()))  
	lescript.write("%s.setFresnelTrans(%f)\n"%(matvar,lemat.getFresnelTrans()) ) 
	lescript.write("%s.setFresnelTransFac(%f)\n"%(matvar,lemat.getFresnelTransFac()) ) 
	lescript.write("%s.setHaloSeed(%d)\n"%(matvar,lemat.getHaloSeed()))  
	lescript.write("%s.setHaloSize(%f)\n"%(matvar,lemat.getHaloSize()))  
	lescript.write("%s.setHardness(%d)\n"%(matvar,lemat.getHardness()))  
	lescript.write("%s.setIOR(%f)\n"%(matvar,lemat.getIOR())) 
	#list of 3 float
	lescript.write("%s.setMirCol(%s)\n"%(matvar,lemat.getMirCol()))
	lescript.write("%s.setMode(%d)\n"%(matvar,lemat.getMode())  )
	lescript.write("%s.setName('%s')\n"%(matvar,lemat.getName()) ) 
	lescript.write("%s.setNFlares(%d)\n"%(matvar,lemat.getNFlares()) ) 
	lescript.write("%s.setNLines(%d)\n"%(matvar,lemat.getNLines())  )
	lescript.write("%s.setNRings(%d)\n"%(matvar,lemat.getNRings())  )
	lescript.write("%s.setNStars(%d)\n"%(matvar,lemat.getNStars()))  
	lescript.write("%s.setRayMirr(%f)\n"%(matvar,lemat.getRayMirr()))  
	#lescript.write("lemat.setRayMirrDepth(%d)\n"%(matvar,lemat.getRayMirrDepth() ))  
	lescript.write("%s.setRef(%f)\n"%(matvar,lemat.getRef()))   
	lescript.write("%s.setRGBCol(%s)\n"%(matvar,lemat.getRGBCol()))  
	lescript.write("%s.setRms(%f)\n"%(matvar,lemat.getRms()) ) 
	lescript.write("%s.setRoughness(%f)\n"%(matvar,lemat.getRoughness()) ) 
	lescript.write("%s.setSpec(%f)\n"%(matvar,lemat.getSpec()) ) 
	lescript.write("%s.setSpecCol(%s)\n"%(matvar,lemat.getSpecCol()))
	lescript.write("%s.setSpecTransp(%f)\n"%(matvar,lemat.getSpecTransp()) ) 
	lescript.write("%s.setSubSize(%f)\n"%(matvar,lemat.getSubSize()))  
	lescript.write("%s.setTransDepth(%d)\n"%(matvar,lemat.getTransDepth()) ) 
	lescript.write("%s.setTranslucency(%f)\n"%(matvar,lemat.getTranslucency()) ) 
	lescript.write("%s.setZOffset(%f)\n"%(matvar,lemat.getZOffset()))  
	lescript.write("%s.materials.append(%s)\n"%(meshradical, matvar))
##"lemat.setIpo(ipo)\n"% 
##
###Assign a Blender Texture object to slot number 'number'
##"lemat.setTexture(index, texture, texco, mapto)\n"%getTextures()  
###from one of the shaders in Material.Shaders dict. 
###"lemat.setSpecShader(%d)\n"%lemat.getSpecShader(specShader)
##
##
##


def exportPython(filename):

  #
  # List selected meshes
  #
  lesobjets = Blender.Object.GetSelected()
  lesmesh = []
  for ob in lesobjets:
    if ob.getType()=="Mesh":
      lesmesh.append(ob)
  #
  # Build object name (add menu entry) from filename
  #
  objectname =  Blender.sys.splitext(Blender.sys.basename(filename))[0]
  if objectname.find('add') == 0:
    objectname = objectname [3:]

  #
  # Create or ReCreate output python file
  #
  lescript = open (filename,"w")

  #
  # write file header
  #
  lescript.write("#!BPY\n")
  lescript.write("\"\"\"\n")
  lescript.write("Name: '"+objectname+"'\n")
  lescript.write("Blender: 237\n")
  lescript.write("Group: 'Add'\n")
  lescript.write("Tooltip: 'Add a "+objectname+" to current scene'\n")
  lescript.write("\"\"\"\n")

  lescript.write("import Blender\n")
  lescript.write("from Blender import *\n")
  lescript.write("def computeName(radical):\n")
  lescript.write("  nextname=\"\"\n")
  lescript.write("  try:\n")
  lescript.write("    ob = Blender.Object.Get(radical)\n")
  lescript.write("    ii = 1\n")
  lescript.write("    while (ob != None) and (ii<= 999):\n")
  lescript.write("      essai=\"%s.%03d\"%(radical,ii)\n")
  lescript.write("      try:\n")
  lescript.write("        ob = Blender.Object.Get(essai)\n")
  lescript.write("      except:\n")
  lescript.write("        nextname=essai\n")
  lescript.write("        ob = None\n")
  lescript.write("      ii+=1\n")
  lescript.write("  except:\n")
  lescript.write("    nextname=radical\n")
  lescript.write("  return (nextname)\n")
  lescript.write("\n")

  lescript.write("def GetOrCreateMaterial(matname):\n")
  lescript.write("    try:\n") 
  lescript.write("        mat = Blender.Material.Get(matname)\n")
  lescript.write("    except:\n")
  lescript.write("        mat = Blender.Material.Get(matname)\n")
  lescript.write("    return mat\n")
  lescript.write("\n")
  lescript.write("def getMaterialIndex(me, matname):\n")
  lescript.write("  ind, res = 0, 0\n")
  lescript.write("  for mat in me.materials:\n")
  lescript.write("    if mat.getName == matname:\n")
  lescript.write("      res = ind\n")
  lescript.write("    ind = ind+1\n")
  lescript.write("  return res\n")

  
  lescript.write("obname = computeName('"+objectname+"')\n")

  #
  # for each selected mesh
  #
  for ob in lesmesh:
    lamesh = ob.getData ()


    #
    # write python commands that create the mesh
    #
    lescript.write("ob=Blender.Object.New('Mesh', obname)\n")

    #lescript.write("me=Mesh.New(obname)\n")
    lescript.write("me=ob.getData(mesh=True)\n")
    nbmat = 0
    matlist = "["
    for mat in lamesh.materials:
      exportMaterial(mat, lescript, "me", nbmat)
      matlist = matlist + "lemat%d,"%nbmat
      nbmat=nbmat+1
    matlist = matlist[0:len(matlist)-1]+']'
    lescript.write("me.materials = %s\n"%matlist)
    #
    # write python commands that build vertices 
    #
    lesvertices =  lamesh.verts
    for ve in lesvertices:
      #lescript.write("v=NMesh.Vert("+str(ve.co[0])+","+str(ve.co[1])+","+str(ve.co[2])+")\n")
      #lescript.write("me.verts.append(v)\n")
      lescript.write("me.verts.extend(%f, %f, %f)\n"%(ve.co[0], ve.co[1], ve.co[2]))

    #
    # write python commands that build faces and uv_mapping
    #
    lesfaces = lamesh.faces
    firstface = 1
    for fa in lesfaces:
      # build vertex list
      vertlist = "("
      #lescript.write("f=NMesh.Face()\n")
      for ve in fa.v:
        #lescript.write("f.v.append(me.verts["+str(lamesh.verts.index(ve))+"])\n")
        vertlist = vertlist+"me.verts[%d],"%(lamesh.verts.index(ve))
      vertlist = vertlist[0:len(vertlist)-1]+')'

      lescript.write("me.faces.extend%s\n"%vertlist)

      if (lamesh.hasFaceUV()) and (firstface == 1): lescript.write("me.faceUV=1\n")
      if firstface == 1: firstface = 0

      if lamesh.hasFaceUV():
        #lescript.write("f.uv=[]\n")
	uvcoordlist = "("
        for uvc in fa.uv:
          #lescript.write("f.uv.append("+str(uvc[0])+","+str(uvc[1])+")\n")
	  uvcoordlist = uvcoordlist + "Blender.Mathutils.Vector(%f, %f),"%(uvc[0],uvc[1])
        uvcoordlist = uvcoordlist[0:len(uvcoordlist)-1]+')'
	
	lescript.write("me.faces[len(me.faces)-1].uv=%s\n"%uvcoordlist)
        #lescript.write("me.faces[len(me.faces)-1].flag=%s\n"%fa.flag)
        lescript.write("me.faces[len(me.faces)-1].mode=%s\n"%fa.mode)
        lescript.write("me.faces[len(me.faces)-1].transp=%s\n"%fa.transp)
      lescript.write("me.faces[len(me.faces)-1].mat=getMaterialIndex(me,'%s')\n"%lamesh.materials[fa.mat].getName())
      #lescript.write("me.faces.append(f)\n")

    #lescript.write("ob=NMesh.PutRaw(me)\n")
    lescript.write("Blender.Scene.getCurrent().link(ob)\n")

    #
    # write python commands that create vertex group 
    #
    groupNames = lamesh.getVertGroupNames() 
    for nam in groupNames:
        lescript.write("me.addVertGroup('%s')\n"% nam)
        for ind_wei in lamesh.getVertsFromGroup(nam, 1):
	    lescript.write("vel=[%d]\n"%ind_wei[0])
            lescript.write("me.assignVertsToGroup('%s', vel, %f, Blender.Mesh.AssignModes.ADD)\n"%(nam, ind_wei[1]))


    #
    # write python commands that write size, position and rotation 
    #
    
    lescript.write("ob.setName(obname)\n")
    lescript.write("ob.setSize(["+str(ob.getSize()[0])+","+str(ob.getSize()[1])+","+str(ob.getSize()[2])+"])\n")
    lescript.write("ob.setLocation(["+str(ob.getLocation()[0]-Blender.Window.GetCursorPos()[0])+"+Blender.Window.GetCursorPos()[0],"+str(ob.getLocation()[1]-Blender.Window.GetCursorPos()[1])+"+Blender.Window.GetCursorPos()[1],"+str(ob.getLocation()[2]-Blender.Window.GetCursorPos()[2])+"+Blender.Window.GetCursorPos()[2]])\n")
    lescript.write("ob.setEuler(["+str(ob.getEuler()[0])+","+str(ob.getEuler()[1])+","+str(ob.getEuler()[2])+"])\n")


  #
  # Close python script file
  #
  lescript.close()
  Blender.UpdateMenus()
  

#
# MAIN
#
   
lesobjets = Blender.Object.GetSelected()
if len(lesobjets) >= 2:
    filename = blender_script_dir+ sys.sep + 'add'+Blender.sys.splitext(Blender.sys.basename(Blender.Get('filename')))[0] +'.py'
    Blender.Window.FileSelector(exportPython, "Export Python", filename)
elif len(lesobjets) == 1:
    filename = blender_script_dir+ sys.sep + 'add'+Blender.Object.GetSelected()[0].name +'.py'
    Blender.Window.FileSelector(exportPython, "Export Python", filename)
else:
    error_txt = "Error%t|Select one or several meshes before running this script"
    Blender.Draw.PupMenu(error_txt)
    print error_txt

